from rest_framework import mixins
from rest_framework.permissions import IsAuthenticated
from rest_framework.viewsets import GenericViewSet

from testapp.api import serializers
from testapp.models import MySingleSignalModel


class MySingleSignalModelViewSet(mixins.ListModelMixin, GenericViewSet):
    permission_classes = [IsAuthenticated]
    serializer_class = serializers.MySingleSignalModelSerializer

    def get_queryset(self):
        return MySingleSignalModel.objects.visible_for(self.request.user)



from crispy_forms.helper import FormHelper
from crispy_forms.layout import Submit, Layout, Div, Fieldset, HTML
from django import forms
from django.utils.translation import gettext_lazy as _


class AdminCrispyForm(forms.Form):
    """
    Base crispy form to be used in custom views within the django admin.
    """
    section_title = _('Kein Titel gesetzt')
    button_label = _('Speichern')

    def __init__(self, *args, **kwargs):
        super().__init__(*args, **kwargs)

        # Build fieldset
        fieldset_list = ['']
        for field in self.fields:
            fieldset_list.append(Div(field, css_class='form-row field-name'))

        # Crispy
        self.helper = FormHelper()
        self.helper.form_method = 'post'
        self.helper.add_input(Submit('submit', self.button_label, css_class="button btn-primary"))
        self.helper.layout = Layout(
            Div(
                Div(
                    HTML(f'<h2>{self.section_title}</h2>'),
                    Fieldset(*fieldset_list),
                    css_class='module aligned'
                ), css_class='custom-form'
            ),
        )



from collections import namedtuple, OrderedDict
from typing import Any


def get_namedtuple_choices(name, choices_tuple):
    # todo copied code -> test if it works
    """
    Made by Ross Crawford-d'Heureuse.

    Factory function for quickly making a namedtuple suitable for use in a
    Django model as a choices attribute on a field. It will preserve order.

    Usage::

        class MyModel(models.Model):
            COLORS = get_namedtuple_choices('COLORS', (
                (0, 'black', 'Black'),
                (1, 'white', 'White'),
            ))
            colors = models.PositiveIntegerField(choices=COLORS)

        >>> MyModel.COLORS.black
        0
        >>> MyModel.COLORS.get_choices()
        [(0, 'Black'), (1, 'White')]

        class OtherModel(models.Model):
            GRADES = get_namedtuple_choices('GRADES', (
                ('FR', 'fr', 'Freshman'),
                ('SR', 'sr', 'Senior'),
            ))
            grade = models.CharField(max_length=2, choices=GRADES)

        >>> OtherModel.GRADES.fr
        'FR'
        >>> OtherModel.GRADES.get_choices()
        [('fr', 'Freshman'), ('sr', 'Senior')]
    """

    class Choices(namedtuple(name, [name for val, name, desc in choices_tuple])):
        __slots__ = ()
        _choices = tuple([desc for val, name, desc in choices_tuple])

        def get_choices(self):
            return list(zip(tuple(self), self._choices))

        def get_choices_dict(self):
            """
            Return an ordered dict of key and their values
            must be ordered correctly as there are items that depend on the key
            order
            """
            choices = OrderedDict()
            for k, v in self.get_choices():
                choices[k] = v
            return choices

        def get_all(self):
            for val, name, desc in choices_tuple:
                yield val, name, desc

        def get_choices_tuple(self):
            return choices_tuple

        def get_values(self):
            values = []
            for val, name, desc in choices_tuple:
                if isinstance(val, type([])):
                    values.extend(val)
                else:
                    values.append(val)
            return values

        def get_value_by_name(self, input_name):
            for val, name, desc in choices_tuple:
                if name == input_name:
                    return val
            return False

        def get_desc_by_value(self, input_value):
            for val, name, desc in choices_tuple:
                if val == input_value:
                    return desc
            return False

        def get_name_by_value(self, input_value):
            for val, name, desc in choices_tuple:
                if val == input_value:
                    return name
            return False

        def is_valid(self, selection):
            for val, name, desc in choices_tuple:
                if val == selection or name == selection or desc == selection:
                    return True
            return False

    return Choices._make([val for val, name, desc in choices_tuple])


def get_value_from_tuple_by_key(choices: tuple, key) -> Any:
    """
    Fetches the tuple value by a given key
    Useful for getting the name of a key from a model choice tuple of tuples.
    Usage: project_type_a_name = get_value_from_tuple_by_key(PROJECT_TYPE_CHOICES, PROJECT_TYPE_A)
    """
    try:
        return dict(choices)[key]
    except KeyError:
        return '-'


def get_key_from_tuple_by_value(choices: tuple, value) -> Any:
    """
    Fetches the tuple key by a given value
    Useful for getting the key of a value from a model choice tuple of tuples.
    Usage: project_type_a_name = get_value_from_tuple_by_key(PROJECT_TYPE_CHOICES, 'Budget-Project')
    """
    try:
        return [x[0] for x in choices if x[1] == value][0]
    except IndexError:
        return '-'



# Generated by Django 3.1 on 2021-04-19 14:16

from django.db import migrations, models
import django.db.models.deletion


class Migration(migrations.Migration):
    dependencies = [
        ('testapp', '0002_auto_210407'),
    ]

    operations = [
        migrations.CreateModel(
            name='ModelWithFkToSelf',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('parent', models.ForeignKey(on_delete=django.db.models.deletion.CASCADE, related_name='children',
                                             to='testapp.modelwithfktoself')),
            ],
        ),
        migrations.AlterField(
            model_name='foreignkeyrelatedmodel',
            name='single_signal',
            field=models.ForeignKey(on_delete=django.db.models.deletion.CASCADE,
                                    related_name='foreign_key_related_models', to='testapp.mysinglesignalmodel'),
        ),
        migrations.AlterField(
            model_name='modelwithfktoself',
            name='parent',
            field=models.ForeignKey(blank=True, null=True, on_delete=django.db.models.deletion.CASCADE,
                                    related_name='children', to='testapp.modelwithfktoself'),
        ),
        migrations.CreateModel(
            name='ModelWithOneToOneToSelf',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('peer', models.OneToOneField(blank=True, null=True, on_delete=django.db.models.deletion.CASCADE,
                                              related_name='related_peer', to='testapp.modelwithonetoonetoself')),
            ],
        ),
    ]



from crispy_forms.helper import FormHelper
from crispy_forms.layout import Submit
from django.utils.translation import gettext_lazy as _


class CrispyLayoutFormMixin:
    """
    Styles the form in bootstrap style
    """

    def __init__(self, *args, **kwargs):
        # Crispy
        self.helper = FormHelper()
        self.helper.form_class = 'form-horizontal form-bordered form-row-stripped'
        self.helper.form_method = 'post'
        self.helper.add_input(Submit('submit_button', _('Speichern')))
        self.helper.form_tag = True
        self.helper.label_class = 'col-md-3'
        self.helper.field_class = 'col-md-9'
        self.helper.label_size = ' col-md-offset-3'

        super().__init__(*args, **kwargs)



import datetime
import os

from django.contrib.humanize.templatetags.humanize import intcomma
from django.template.defaultfilters import floatformat

from ai_django_core.utils import datetime_format


def distinct(not_distinct_list: list) -> list:
    """
    Returns a list with no duplicate elements
    """
    return list(set(not_distinct_list))


def slugify_file_name(file_name: str, length: int = 40) -> str:
    """
    Slugify the given file name
    """
    from django.template.defaultfilters import slugify
    from django.utils.encoding import smart_str
    name, ext = os.path.splitext(file_name)
    name = smart_str(slugify(name).replace('-', '_'))
    ext = smart_str(slugify(ext))
    result = '%s%s%s' % (name[:length], "." if ext else "", ext)
    return result


def smart_truncate(text: str, max_length: int = 100, suffix: str = '...') -> str:
    """
    Returns a string of at most `max_length` characters, cutting
    only at word-boundaries. If the string was truncated, `suffix`
    will be appended.
    In comparison to djangos defaultfilter `truncatechars` this method does NOT break words and you
    can choose a custom suffix.
    # todo write test & rename (?) & add filter?
    """

    if text is None:
        return ''

    # Return the string itself if length is smaller or equal to the limit
    if len(text) <= max_length:
        return text

    # Cut the string
    value = text[:max_length]

    # Break into words and remove the last
    words = value.split(' ')[:-1]

    # Join the words and return
    return ' '.join(words) + suffix


def float_to_string(value: float, replacement: str = "0,00") -> str:
    """
    Converts a float to a properly formatted string value
    """
    return ("%.2f" % value).replace('.', ',') if value is not None else replacement


def date_to_string(value: datetime.date, replacement: str = "-", str_format: str = "%d.%m.%Y") -> str:
    return value.strftime(str_format) if value is not None else replacement


def datetime_to_string(value: datetime.datetime, replacement: str = "-", str_format: str = "%d.%m.%Y %H:%M") -> str:
    return datetime_format(value, str_format) if value is not None else replacement


def number_to_string(value, decimal_digits: int = 0, replacement: str = "-") -> str:
    return intcomma(floatformat(value, decimal_digits)) if value is not None else replacement


def string_or_none_to_string(value, replacement: str = "-") -> str:
    return value if value is not None else replacement


def encode_to_xml(text: str) -> str:
    text_str = str(text)
    text_str = text_str.replace('&', '&amp;')
    text_str = text_str.replace('<', '&lt;')
    text_str = text_str.replace('>', '&gt;')

    return text_str



import datetime
from calendar import monthrange
from typing import Union

import pytz
from dateutil.relativedelta import relativedelta
from django.conf import settings
from django.utils import timezone
from django.utils.translation import gettext_lazy as _
from pytz import UnknownTimeZoneError


class DateHelper:
    """
    Constants to use for django ORMs `__weekday` lookup to avoid usage of integers directly.
    Unfortunately python's calendar weekdays are not equivalent to the database ones.
    """
    ORM_SUNDAY = 1
    ORM_MONDAY = 2
    ORM_TUESDAY = 3
    ORM_WEDNESDAY = 4
    ORM_THURSDAY = 5
    ORM_FRIDAY = 6
    ORM_SATURDAY = 7


def tz_today(str_format=None):
    """
    Returns either a timezone-aware today object or string. Depending if the `str_format` param is set.
    :param str_format: If set, formats the date object to a string. Pass strftime-compatible value
    :return:
    """
    if settings.USE_TZ:
        date = timezone.now().date()
    else:
        date = datetime.datetime.now().date()

    if str_format:
        return date.strftime(str_format)
    return date


def add_months(source_date: Union[datetime.date, datetime.datetime], months: Union[int, float]) -> \
        Union[datetime.date, datetime.datetime]:
    return source_date + relativedelta(months=months)


def add_days(source_date: Union[datetime.date, datetime.datetime], days: int) -> \
        Union[datetime.date, datetime.datetime]:
    return source_date + datetime.timedelta(days=days)


def add_minutes(source_datetime: datetime.datetime, minutes: int) -> Union[datetime.date, datetime.datetime]:
    return source_datetime + relativedelta(minutes=minutes)


def get_next_month() -> datetime.date:
    return add_months(tz_today(), 1)


def first_day_of_month(source_date: datetime.date) -> datetime.date:
    return datetime.date(source_date.year, source_date.month, 1)


def get_formatted_date_str(source_date: Union[datetime.date, datetime.datetime]) -> str:
    return source_date.strftime('%d.%m.%Y')


def get_time_from_seconds(seconds: int) -> str:
    if seconds < 0:
        raise ValueError(_('Seconds have to be a positive value.'))
    hours = seconds // 3600
    minutes = (seconds - (hours * 3600)) // 60
    seconds = seconds - ((hours * 3600) + (minutes * 60))
    return "%02d:%02d:%02d" % (hours, minutes, seconds)


def datetime_format(target_datetime: datetime.datetime, dt_format: str) -> str:
    """
    Uses strftime, but considers timezone (only for datetime objects)
    """
    try:
        dt_format = target_datetime.astimezone(tz=pytz.timezone(settings.TIME_ZONE)).strftime(dt_format)
    except UnknownTimeZoneError:
        dt_format = target_datetime.strftime(dt_format)
    return dt_format


def get_start_and_end_date_from_calendar_week(year: int, calendar_week: int) -> (datetime.date, datetime.date):
    """
    Returns the first and last day of a given calendar week
    """
    monday = datetime.datetime.strptime(f'{year}-{calendar_week}-1', "%Y-%W-%w").date()
    return monday, monday + datetime.timedelta(days=6.9)


def get_next_calendar_week(compare_date: datetime.date) -> int:
    """
    Returns the next calendar week as an integer
    """
    day_in_one_week = compare_date + datetime.timedelta(days=(7 - compare_date.weekday()))
    return day_in_one_week.isocalendar()[1]


def next_weekday(given_date: datetime.date, weekday: int) -> datetime.date:
    """
    Returns the next date of the given weekday
    For example: next_weekday(d, calendar.MONDAY) would return the next monday starting at date "given_date"
    """
    days_ahead = weekday - given_date.weekday()
    if days_ahead <= 0:
        days_ahead += 7
    return given_date + datetime.timedelta(days_ahead)


def date_month_delta(start_date: datetime.date, end_date: datetime.date) -> float:
    """
    Calculates the number of months lying between two dates.
    So from April 15th to May 1st it's 0.5 months.
    Attention: `end_date` will be excluded in the result (outer border)
    """
    # If `start_date` is greater, this logic doesn't make any sense
    if start_date > end_date:
        raise NotImplementedError('Start date > end date')

    # Calculate date difference between dates
    date_diff = (end_date - start_date).days

    # Iteration variable
    delta = 0
    iter_date = start_date
    # Loop until all days are processed
    while date_diff > 0:
        # Get days of month we are currently looking at
        iter_month, iter_month_days = monthrange(iter_date.year, iter_date.month)
        # Calculate how many days are left to end of this month
        days_to_month_end = min((iter_month_days - (iter_date.day - 1)), date_diff)
        # Add percentage of the month these days cover to return variable
        delta += days_to_month_end / iter_month_days
        # Reduce leftover days by the amount we already processed
        date_diff -= days_to_month_end

        # Set iteration date to first of next month
        iter_date += relativedelta(months=1, day=1)

    # Return data
    return delta



from django.urls import resolve


class AdminCreateFormMixin:
    """
    Mixin to easily use a different form for the create case (in comparison to "edit") in the django admin
    Logic copied from `django.contrib.auth.admin.UserAdmin`
    """
    add_form = None

    def get_form(self, request, obj=None, **kwargs):
        defaults = {}
        if obj is None:
            defaults['form'] = self.add_form
        defaults.update(kwargs)
        return super().get_form(request, obj, **defaults)


class AdminNoInlinesForCreateMixin:
    """
    Avoid showing any inline admins when being in the create case of a record
    """

    def get_inline_instances(self, request, obj=None):
        if obj is None:
            return []
        return super().get_inline_instances(request, obj)


class AdminRequestInFormMixin:
    """
    Mixin to add the current request to a form used in the django admin
    """

    def get_form(self, request, obj=None, **kwargs):
        form = super().get_form(request, obj, **kwargs)
        form.request = request
        return form


class FetchParentObjectInlineMixin:
    """
    Fetches the parent object via the URL resolver and makes it available throughout the entire class.
    Attention: Use only in inline admin classes.
    """
    parent_object = None

    @staticmethod
    def _resolve_url(request):
        return resolve(request.path_info)

    def get_parent_object_from_request(self, request):
        resolved = self._resolve_url(request)
        if resolved.kwargs:
            return self.parent_model.objects.get(pk=resolved.kwargs.get('object_id', None))
        return None

    def get_formset(self, request, obj=None, **kwargs):
        # Fetch parent object - cannot be done in the init method because request is not available there
        self.parent_object = self.get_parent_object_from_request(request)
        return super().get_formset(request, obj, **kwargs)


class FetchObjectMixin:
    """
    Fetches the current object via the URL resolver and makes it available throughout the entire class.
    """

    def get_object_from_request(self, request):
        resolved = resolve(request.path_info)
        if resolved.kwargs:
            return self.model.objects.get(pk=resolved.kwargs.get('object_id', None))
        return None


class CommonInfoAdminMixin:
    """
    Mixin to be used in a django model admin class.
    Sets all four `CommonInfo` attributes to "readonly" and sets the creator / last modifier on form save.
    """

    def get_readonly_fields(self, request, obj=None):
        return super().get_readonly_fields(request, obj) + ('created_by', 'lastmodified_by', 'created_at',
                                                            'lastmodified_at')

    def save_form(self, request, form, change):
        if form.instance and request.user:
            if not form.instance.id:
                form.instance.created_by = request.user
            form.instance.lastmodified_by = request.user

        return super().save_form(request, form, change)



from django.contrib.auth.models import AbstractUser
from rest_framework import status
from rest_framework.response import Response
from rest_framework.test import APIRequestFactory, force_authenticate
from rest_framework.viewsets import GenericViewSet


class BaseViewSetTestMixin:
    """
    Base test mixin to lower the pain of testing API calls. Focuses on ViewSets.
    """
    # todo documentation
    default_api_user = None
    view_class = None

    @classmethod
    def setUpTestData(cls) -> None:
        super().setUpTestData()

        # Create test factory
        cls.factory = APIRequestFactory()

    def setUp(self) -> None:
        super().setUp()

        # Create API user with proper permissions
        self.default_api_user = self.get_default_api_user()

    def get_default_api_user(self) -> AbstractUser:
        """
        Method to create the API request user in.
        """
        raise NotImplementedError

    def validate_authentication_required(self, *, url: str, method: str, view: str):
        """
        Helper method to quickly ensure that a given view needs authorisation.
        """
        response = self.execute_request(
            method=method,
            url=url,
            viewset_kwargs={method: view})

        self.assertIn(response.status_code, [status.HTTP_401_UNAUTHORIZED, status.HTTP_403_FORBIDDEN])

    def execute_request(self, *, url, view_kwargs=None, method='get', data=None, view_class=None, user=None,
                        viewset_kwargs=None, data_format='json') -> Response:
        """
        Helper method which wraps all relevant setup to execute a request to the backends api

        view_kwargs: Kwargs from the URL (need to be passed manually to the view class as well)
        method: Request method (get, post, delete...)
        data: Dict of data to pass to the request, usually for post, put, patch
        view_class: Will overwrite the classes default view_class if set
        user: User to authenticate the request with. If no user is set, there will be no authentication
        viewset_kwargs: ViewSets need a dict to tell it, what action we want to execute
        data_format: Format of the sent data, defaults to JSON, might be "multipart" etc.
        """

        # Set proper default fallback value for dicts
        if view_kwargs is None:
            view_kwargs = {}
        if viewset_kwargs is None:
            viewset_kwargs = {}

        # Create request
        request = getattr(self.factory, method)(path=url, data=data, format=data_format)

        # Authentication
        if user:
            force_authenticate(request, user=user)

        # Check if we want to override the view class for this single call
        view_class = view_class if view_class else self.view_class

        # Add viewset kwargs if we have a viewset here
        if isinstance(view_class(), GenericViewSet):
            view = view_class.as_view(viewset_kwargs)
        else:
            view = view_class.as_view()

        # Get response
        response = view(request, **view_kwargs)

        # Return response
        return response



from django.contrib.auth.base_user import AbstractBaseUser
from django.contrib.auth.models import AnonymousUser
from django.contrib.messages.middleware import MessageMiddleware
from django.contrib.messages.storage.fallback import FallbackStorage
from django.contrib.sessions.middleware import SessionMiddleware
from django.test import RequestFactory


class ClassBasedViewTestMixin:
    """
    Helper mixin for easy testing of Django views
    """
    factory_class = RequestFactory
    view_class = None

    @staticmethod
    def _authentication(request, user):
        request.user = user if user else AnonymousUser()

    def _get_response(self, method, user, data, url_params=None, *args, **kwargs):
        """Returns response."""

        # Catch case that URL does not get any params passed to
        if not url_params:
            url_params = {}

        # Create request
        factory = self.factory_class()
        req_kwargs = {}
        if data:
            req_kwargs.update({'data': data})
        req = getattr(factory, method)('/', **req_kwargs)

        # Annotate a request object with a session
        middleware = SessionMiddleware()
        middleware.process_request(req)
        req.session.save()

        # Authenticate user
        self._authentication(req, user)

        # Setup messages
        messages = FallbackStorage(req)
        setattr(req, '_messages', messages)

        # Call view
        return self.view_class.as_view()(req, *args, **url_params)

    def get(self, user=None, data=None, url_params=None, *args, **kwargs):
        """Returns response for a GET request."""
        return self._get_response('get', user, data, url_params, *args, **kwargs)

    def post(self, user=None, data=None, url_params=None, *args, **kwargs):
        """Returns response for a POST request."""
        return self._get_response('post', user, data, url_params, *args, **kwargs)

    def delete(self, user=None, data=None, url_params=None, *args, **kwargs):
        """Returns response for a DELETE request."""
        return self._get_response('delete', user, data, url_params, *args, **kwargs)


class RequestProviderMixin:
    """
    Mixin that provides a method which returns a django request.
    Supposed to be used in unittests that require a request object.
    Also adds the message middleware so django messages can be tested.
    """

    @staticmethod
    def get_request(user: AbstractBaseUser = None):
        """
        Creates and returns a django request.
        """
        # Create test request
        factory = RequestFactory()
        request = factory.get('/')
        request.user = user

        # Annotate a request object with a session
        session_middleware = SessionMiddleware()
        session_middleware.process_request(request)
        request.session.save()

        # Annotate a request object with messages
        message_middleware = MessageMiddleware()
        message_middleware.process_request(request)
        request.session.save()

        return request



# Generated by Django 3.1.2 on 2021-03-26 09:37
from django.conf import settings
from django.db import migrations, models
import django.db.models.deletion


class Migration(migrations.Migration):

    dependencies = [
        ('testapp', '0001_initial'),
    ]

    operations = [
        migrations.CreateModel(
            name='ForeignKeyRelatedModel',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('single_signal', models.ForeignKey(on_delete=django.db.models.deletion.CASCADE, to='testapp.mysinglesignalmodel')),
            ],
        ),
        migrations.CreateModel(
            name='CommonInfoBasedModel',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('created_at',
                 models.DateTimeField(db_index=True, default=django.utils.timezone.now, verbose_name='Erstellt am')),
                ('lastmodified_at', models.DateTimeField(db_index=True, default=django.utils.timezone.now,
                                                         verbose_name='Zuletzt geändert am')),
                ('value', models.PositiveIntegerField(default=0)),
                ('created_by', models.ForeignKey(blank=True, null=True, on_delete=django.db.models.deletion.SET_NULL,
                                                 related_name='testapp_commoninfobasedmodel_created',
                                                 to=settings.AUTH_USER_MODEL, verbose_name='Erstellt von')),
                ('lastmodified_by',
                 models.ForeignKey(blank=True, null=True, on_delete=django.db.models.deletion.SET_NULL,
                                   related_name='testapp_commoninfobasedmodel_lastmodified',
                                   to=settings.AUTH_USER_MODEL, verbose_name='Zuletzt geändert von')),
            ],
            options={
                'abstract': False,
            },
        ),
    ]



from django.test import TestCase

from ai_django_core.utils.string import slugify_file_name


class UtilsStringTest(TestCase):

    def test_slugify_file_name_regular(self):
        filename = 'hola and hello.txt'
        slug = slugify_file_name(filename)
        self.assertEqual(slug, 'hola_and_hello.txt')

    def test_slugify_file_name_nothing_to_slugify(self):
        filename = 'hola.txt'
        slug = slugify_file_name(filename)
        self.assertEqual(slug, filename)

    def test_slugify_file_name_max_length(self):
        filename = 'a very long filename.txt'
        slug = slugify_file_name(filename, 6)
        self.assertEqual(slug, 'a_very.txt')



#!/usr/bin/env python
import ast
import re

from setuptools import find_packages, setup

# Dependencies
requirements = [
    'Django>=2.0.13',
    'bleach>=1.4.3',
    'python-dateutil>=2.5.3',
    'html2text>=2020.1.16',
]

# Parse version
_version_re = re.compile(r"__version__\s+=\s+(.*)")
with open("ai_django_core/__init__.py", "rb") as f:
    version = str(
        ast.literal_eval(_version_re.search(f.read().decode("utf-8")).group(1))
    )

setup(
    name='ai-django-core',
    version=version,
    author='Ambient Innovation: GmbH',
    author_email='hello@ambient-innovation.com',
    packages=find_packages(),
    url='https://github.com/ambient-innovation/ai-django-core',
    project_urls={
        'Documentation': 'https://ai-django-core.readthedocs.io/en/latest/index.html',
        'Maintained by': 'https://ambient-innovation.com/',
    },
    include_package_data=True,
    license="The MIT License (MIT)",
    description='Lots of helper functions and useful widgets.',
    long_description=open('README.md').read(),
    long_description_content_type='text/markdown',
    classifiers=[
        "Development Status :: 5 - Production/Stable",
        "Environment :: Web Environment",
        "Framework :: Django",
        "Framework :: Django :: 2.2",
        "Framework :: Django :: 3.1",
        "Framework :: Django :: 3.2",
        "Intended Audience :: Developers",
        "License :: OSI Approved :: MIT License",
        "Natural Language :: English",
        "Operating System :: OS Independent",
        "Programming Language :: Python",
        "Programming Language :: Python :: 3",
        "Programming Language :: Python :: 3.6",
        "Programming Language :: Python :: 3.7",
        "Programming Language :: Python :: 3.8",
        "Programming Language :: Python :: 3.9",
        "Topic :: Utilities",
    ],
    zip_safe=False,
    install_requires=requirements,
    extras_require={
        'dev': [
            'freezegun',
            'pytest-cov',
            'pytest-django',
            'pytest-mock',
            'gevent',
            'sphinx',
            'recommonmark',
            'sphinx-rtd-theme',
            'm2r2',
        ],
        'docs': [
            'recommonmark',
            'sphinx-rtd-theme',
            'm2r2',
        ],
        'drf': [
            'djangorestframework>=3.8.2',
        ],
        'graphql': [
            'graphene-django>=2.2.0',
            'django-graphql-jwt>=0.2.1',
        ],
        'view-layer': [
            'django-crispy-forms>=1.4.0',
        ]
    },
)



from django.test import TestCase

from ai_django_core.utils import get_value_from_tuple_by_key, get_key_from_tuple_by_value


class UtilsNamedTupleTest(TestCase):

    @classmethod
    def setUpClass(cls):
        super().setUpClass()

        cls.MY_CHOICE_ONE = 1
        cls.MY_CHOICE_TWO = 2
        cls.MY_CHOICE_THREE = 3

        cls.MY_CHOICE_LIST = (
            (cls.MY_CHOICE_ONE, "Choice 1"),
            (cls.MY_CHOICE_TWO, "Choice 2"),
            (cls.MY_CHOICE_THREE, "Choice 3"),
        )

    def test_get_value_from_tuple_by_key_found(self):
        self.assertEqual(get_value_from_tuple_by_key(self.MY_CHOICE_LIST, self.MY_CHOICE_TWO), 'Choice 2')

    def test_get_value_from_tuple_by_key_not_found(self):
        self.assertEqual(get_value_from_tuple_by_key(self.MY_CHOICE_LIST, 99), '-')

    def test_get_key_from_tuple_by_value_found(self):
        self.assertEqual(get_key_from_tuple_by_value(self.MY_CHOICE_LIST, 'Choice 2'), self.MY_CHOICE_TWO)

    def test_get_key_from_tuple_by_value_not_found(self):
        self.assertEqual(get_key_from_tuple_by_value(self.MY_CHOICE_LIST, 'Something odd'), '-')
